#!/usr/bin/perl
#------------------------------------------------------------------
# YOU ARE COMING TO THE DESTINATION ALMOST. PLEASE DO A LITTLE EFFORT.
#
# How to setup search form of htags
# =================================
#
# You must setup HTTP server so that this script can be executed
# as a CGI script. This setup depends on the HTTP server which you
# are using.
#
# If you are using Apache HTTP Server [http://httpd.apache.org/],
# you may set it up by adding an entry to the configuration file
# [http.conf] like follows:
#
# +-----------------------------------------
# |...
# |<Directory /usr/local/apache/htdocs/tour> <= depends your environment
# |    Options +ExecCGI
# |    AddHandler cgi-script .cgi
# |</Directory>
# |...
# +-----------------------------------------
#
# If you are using other systems, please consult the manuals.
#------------------------------------------------------------------
#
# Copyright (c) 1997, 2004, 2006, 2008 Tama Communications Corporation
#
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without
# modifications, as long as this notice is preserved.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
print "Content-type: text/html\n\n";
print "<html>\n";
print "<head>\n<style type='text/css'>\n";
print "body { color: #191970; background-color: #f5f5dc; }\n";
print "a.visited { color: gray; }\n";
print ".title { color: #cc0000; }\n";
print ".error { color: red; }\n";
print ".fname { color: blue; }\n";
print "</style>\n</head>\n";
print "<body>\n";
$htmlbase = $ENV{'HTTP_REFERER'};
if (!$htmlbase) {
	&print_error_msg("Your browser doesn't send HTTP_REFERER.");
	exit 0;
}
$htmlbase =~ s!/[^\/]+$!!;		# remove file name.
$htmlbase =~ s!/defines$!!;
$htmlbase =~ s!/S$!!;
$htmlbase =~ s!/$!!;
$retlink = "<a href='$htmlbase/mains.html'>[return]</a>";
if (! -x '/usr/bin/global') {
	&print_error_msg("Server side command not found. $retlink");
	exit 0;
}
@pairs = split (/&/, $ENV{'QUERY_STRING'});
foreach $p (@pairs) {
	($name, $value) = split(/=/, $p);
	$value =~ tr/+/ /;
	$value =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack("C", hex($1))/eg;
	$form{$name} = $value;
}
if (!defined($form{'id'})) {
	&print_error_msg("No form id. Please regenerate hypertext with htmake.\n");
	exit 0;
}
if ($form{'pattern'} eq '') {
	&print_error_msg("Pattern not specified. $retlink");
	exit 0;
}
$pattern = $form{'pattern'};
$flag = '';
$words = 'definitions';
if ($form{'type'} eq 'reference') {
	$flag = 'r';
	$words = 'references';
} elsif ($form{'type'} eq 'symbol') {
	$flag = 's';
	$words = 'symbols';
} elsif ($form{'type'} eq 'path') {
	$flag = 'P';
	$words = 'paths';
} elsif ($form{'type'} eq 'grep') {
	$flag = 'g';
	$words = 'patterns';
} elsif ($form{'type'} eq 'idutils') {
	$flag = 'I';
	$words = 'patterns';
}
$iflag = '';
if ($form{'icase'}) {
	$iflag = 'i';
}
$oflag = '';
if ($form{'other'} && ($flag eq 'g' || $flag eq 'P')) {
	$oflag = 'o';
}
$CONFILE = '/etc/gtags/htmake.conf';
-f "$CONFILE" and do "$CONFILE" or &alertmissing($CONFILE);
&alertmissing('db') unless (defined($GSPATHDATA));
-f "$GSPATHDATA" and do "$GSPATHDATA" or &alertmissing($GSPATHDATA);
&alertmissing('id') unless (defined($hpath));
# call sanity/security checks here before executing global/setting ENV
$ENV{'GTAGSROOT'} = $srcpath;
$ENV{'GTAGSDBPATH'} = $dbpath;
chdir("$srcpath");
if ($?) {	
	&print_error_msg("Couldn't change directory for $form{'type'} search. $retlink");
	exit 0;
}
#
# Sanity check
# GTAGS and GPATH is indispensable file.
#
$missed = '';
if (! -f 'GTAGS') {
	$missed = 'GTAGS';
} elsif (! -f 'GPATH') {
	$missed = 'GPATH';
} elsif ($flag eq 'r' && ! -f 'GRTAGS') {
	$missed = 'GRTAGS';
} elsif ($flag eq 's' && ! -f 'GSYMS') {
	$missed = 'GSYMS';
} elsif ($flag eq 'I' && ! -f 'ID') {
	$missed = 'ID';
}
if ($missed) {
	&print_error_msg("Tag file ($missed) not found. $retlink");
	exit 0;
}
local(%ctab) = ('&', '&amp;', '<', '&lt;', '>', '&gt;');
#
# fork and exec global(1) to avoid command substitutions in $pattern.
# The --result=ctags-xid print the file id of the path at the head
# of each line.
#
$flags = $flag . $iflag . $oflag;
if ($flags eq '') {
	open(PIPE, "-|") || exec '/usr/bin/global', '--result=ctags-xid', '-e', $pattern;
} else {
	open(PIPE, "-|") || exec '/usr/bin/global', '--result=ctags-xid', '-'.$flags, '-e', $pattern;
}
if ($?) {
	&print_error_msg("Cannot execute global. $retlink");
	exit 0;
}
$pattern =~ s/([&<>])/$ctab{$1}/ge;
print "<h1 class='title'>" . $pattern . "</h1>\n";
print "<p>Following $words are matched to above pattern.</p><hr />\n";
$cnt = 0;
local($tag, $lno, $filename);
print "<pre>\n";
#
# Input format:
#
# fid tag   lno filename
# ---------------------------------------------
# 100 main  32 ./main.c main(argc, argv)
#
while (<PIPE>) {
	$cnt++;
	($fid, $tag, $lno, $filename) = split;
	s/^[0-9]+[ \t]+//;
	s/([&<>])/$ctab{$1}/ge;
	s!(^[^ \t]+)!<a href='$htmlbase/S/$fid.$suffix#L$lno'>$1<\/a>!;
	print;
}
close(PIPE);
print "</pre>\n";
print "<hr />\n";
if ($cnt == 0) {
	print "<p>Pattern not found. $retlink</p>\n";
}
print "</body>\n</html>\n";
exit 0;

sub alertmissing {
	print "<h1 class='error'>Error</h1>\n";
	if (@_ eq 'id') {
		print "<p>No path data configured for this hypertext source.</p>\n";
		print "<p>Please run htconfig on it to enable this search form.</p>\n";
	} elsif (@_ eq 'db') {
		print "<p>Path database location not defined.</p>\n";
		print "<p>Please set GSPATHDATA in $CONFILE.</p>\n";
	} else {
		print "<p>Unable to read <span class='fname'>@_</span></p>\n";
		print "<p>Please ensure this file exists, and is readable by <br />\n";
		print "the user or group that your cgi scripts run as.</p>\n";
		print "<p>UID: $< or GID: $( </p>\n";
	}
	print "</body>\n</html>\n";
	exit 0;
}

sub print_error_msg {
	print "<h1 class='error'>Error</h1>\n";
	print "<p>@_</p>\n";
	print "</body>\n</html>\n";
}
